@@ -1,6 +1,6 @@
 /* convert.h - Convert documents using LibreOfficeKit
  *
- * Copyright (C) 2015,2016 Olly Betts
+ * Copyright (C) 2015,2016,2025 Olly Betts
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,8 +12,8 @@
 
 extern const char * program;
 
-void * convert_init();
-int convert(void * h_void, bool url,
+void * convert_init(bool verbose);
+int convert(void * h_void, bool url, bool quiet,
 	    const char * input, const char * output,
 	    const char * format = 0, const char * options = 0);
 void convert_cleanup(void * h_void);
